@@ -23,7 +23,7 @@ checkMatrixPackageVersion <- function() {
             "Current Matrix version is ",
             cur.Matrix.version,
             "\n",
-            "Please re-install 'TMB' from source ",
+            "Please re-install 'TMB' from source using install.packages('TMB', type = 'source') ",
             "or ask CRAN for a binary version of 'TMB' matching CRAN's 'Matrix' package"
         )
     }
@@ -32,6 +32,15 @@ checkMatrixPackageVersion <- function() {
 .onLoad <- function(lib, pkg) {
     library.dynam("TMB", pkg, lib)
     checkMatrixPackageVersion()
+    ## Select AD framework (CppAD or TMBad) used by TMB::compile
+    tmb.ad.framework <- getOption("tmb.ad.framework", NULL)
+    if (is.null(tmb.ad.framework))
+        tmb.ad.framework <- Sys.getenv("TMB_AD_FRAMEWORK", "CppAD")
+    options("tmb.ad.framework" = tmb.ad.framework)
+}
+
+.onUnload <- function(libpath) {
+    library.dynam.unload("TMB", libpath)
 }
 
 ## .LastLib <- function(libpath)
@@ -43,5 +52,5 @@ checkMatrixPackageVersion <- function() {
 .onAttach <- function(lib, pkg) {
   exfolder <- system.file("examples", package = "TMB")
   dll <- paste0(exfolder, Sys.getenv("R_ARCH"), "/simple", .Platform$dynlib.ext)
-  if(!file.exists(dll)) runExample("simple", dontrun=TRUE)
+  if(!file.exists(dll)) runExample("simple", dontrun=TRUE, eigen.disable.warnings=FALSE)
 }
